/* $Id: playlistitemsongdb.cpp 154 2007-09-22 00:04:00Z trekkie_lugfl $ */
/***************************************************************************
 *   OpenRadio - RadioMixer                                                *
 *   Copyright (C) 2006-2007 by Jan Boysen                                *
 *   trekkie@media-mission.de                                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "playlistitemsongdb.h"

playListItemSongDB::playListItemSongDB( const unsigned int id,  unsigned int lastPlayedTS, QObject *parent, const char *name  )
 : playListItem( "", parent, name )
{
	lastPlayed = lastPlayedTS;
	songDBId = id;

	QSettings* config = new QSettings();
	songDBHndl = new QHttp( this, "httpSocket" );
	songDBHndl->setHost( config->readEntry( "/radiomixer/network/songDBHostname", "localhost" ) );
	connect( songDBHndl, SIGNAL(done(bool)), this, SLOT(receiveData(bool)));

	songDB = new QHttpRequestHeader( "POST", config->readEntry( "/radiomixer/network/songDBScriptname", "/xmlctrl.pl" ) );
	songDB->setValue( "Host", config->readEntry( "/radiomixer/network/songDBHostname", "localhost" ) );

	delete config;
}

playListItemSongDB::~playListItemSongDB()
{
	delete songDBHndl;
	delete songDB;
}

bool playListItemSongDB::hasCostumBackgroundColor( )
{
	if( state <= Normal )
		return ((QDateTime::currentDateTime().toTime_t() - lastPlayed) < 7200);
	else
		return playListItem::hasCostumBackgroundColor( );
}

QColor playListItemSongDB::getBackgroundColor( )
{
	if( state <= Normal )
	{
		unsigned int playedAgo = QDateTime::currentDateTime().toTime_t() - lastPlayed;

		// calculate the color of the Item
		if( playedAgo < 7200 )
			return Qt::red.dark( 150-(int(playedAgo/720)*10) );
	}
	return playListItem::getBackgroundColor();
}

const QString playListItemSongDB::getId( )
{
	return QString::number(songDBId);
}

void playListItemSongDB::readMeta( )
{
	songDBHndl->request( *songDB, QString("getSonginfo=1&songID="+QString::number(songDBId)).utf8() );
}

void playListItemSongDB::receiveData( bool )
{
	QSettings* config = new QSettings();
	QDomDocument readdata;

	readdata.setContent( songDBHndl->readAll() );
        if( !readdata.isDocument() )
        {
            qWarning( tr("Received an invalid Document") );
        }else
		if( readdata.doctype().name() == "songDBSongInfo" )
		{
			QDomElement dataroot = readdata.documentElement();
			QDomElement song = dataroot.childNodes().item(0).toElement();

			Filename = config->readEntry( "/radiomixer/network/songDBBasePath", "/songs/" )+song.attribute("relPath")+song.attribute("filename");
			setArtist( song.attribute("interpret") );
			setTitle( song.attribute("title") );
			setVote( song.attribute("rated").toInt() );
			setGenre( song.attribute("genre") );
			setLength( song.attribute("length") );
			setPreLength( song.attribute("preTime") );

 			emit refreshed();
			state = Normal;
			emit( ready( this ) );
		}
	delete config;
}

void playListItemSongDB::setFile( QString file )
{
	parseAbsFile(file);
}

void playListItemSongDB::startPlaying( )
{
	playListItem::startPlaying();
	songDBHndl->request( *songDB, QString("updateLastPlayed=1&songID="+QString::number(songDBId)).utf8() );
}

QDomElement playListItemSongDB::toDomElement( QDomDocument* doc )
{
	QDomElement entry = doc->createElement("songdbEntry");
	entry.setAttribute("id", songDBId );

	return entry;
}

void playListItemSongDB::refreshMeta()
{
	readMeta();
}

void playListItemSongDB::load(const unsigned int id )
{
	songDBId = id;
	readMeta();
}
